// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/VetoedFinalState.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Projections/DressedLeptons.hh"
#include "Rivet/Projections/MissingMomentum.hh"
#include "Rivet/Particle.hh"
#include "HepMC/GenEvent.h"

namespace Rivet {
 /// Wee Wemu Wmumu
  class ATLAS_2013_I1190187 : public Analysis {
  //protected:

		public:

		  /// Default constructor
		  ATLAS_2013_I1190187()
		    : Analysis("ATLAS_2013_I1190187")
		  {      
		    setNeedsCrossSection(true);

		  }  

		struct Particles_pT
		{
			  bool operator()( const Particle& lx, const Particle& rx ) const {
			  	 return ( lx.pT() > rx.pT());
			  }
		};


		void init() {
			FinalState fs;

			vector<pair<double, double> > etaRanges_EL;
			etaRanges_EL.push_back(make_pair(-1.37,1.37));
			etaRanges_EL.push_back(make_pair(-2.47, -1.52));
			etaRanges_EL.push_back(make_pair(1.52, 2.47));

			vector<pair<double, double> > etaRanges_MU;
			etaRanges_MU.push_back(make_pair(-2.4,2.4));

			MissingMomentum met(fs);
			addProjection(met, "MET");

			IdentifiedFinalState Photon(fs);
			Photon.acceptIdPair(PID::PHOTON);

			IdentifiedFinalState bare_EL(fs);
			bare_EL.acceptIdPair(PID::ELECTRON);

			IdentifiedFinalState bare_MU(fs);
			bare_MU.acceptIdPair(PID::MUON);

			IdentifiedFinalState neutrinoFS(fs);
			neutrinoFS.acceptNeutrinos();
			addProjection(neutrinoFS, "Neutrinos");

			////////////////////////////////////////////////////////
			//DRESSED LEPTONS
			//	3.arg: 0.1 		= dR(lep,phot)
			//	4.arg: true		= do clustering
			//	7.arg: false	= ignore photons from hadron or tau
			//
 			//////////////////////////////////////////////////////////

			DressedLeptons electronFS(Photon, bare_EL, 0.1,true, etaRanges_EL, 20, false);
			addProjection(electronFS, "ELECTRON_FS");

			DressedLeptons muonFS(Photon, bare_MU, 0.1,true, etaRanges_MU, 20, false);
			addProjection(muonFS, "MUON_FS");

		 	VetoedFinalState jetinput;
			jetinput.addVetoOnThisFinalState(bare_MU);
			jetinput.addVetoOnThisFinalState(neutrinoFS);

			FastJets jetpro(jetinput, FastJets::ANTIKT, 0.4);
			addProjection(jetpro, "jet");

			//book histograms
			_h_Wl1_pT_mumu = bookHisto1D(1,1,2); 
			_h_Wl1_pT_ee = bookHisto1D(1,1,1); 
			_h_Wl1_pT_emu = bookHisto1D(1,1,3); 
			_h_Wl1_pT_inclusive = bookHisto1D(4,1,1);

		}
		
		/// Do the analysis
		void analyze(const Event & e) {

			const double weight = e.weight();

			const  vector<ClusteredLepton>& muonFS = applyProjection<DressedLeptons>(e, "MUON_FS").clusteredLeptons();
			const  vector<ClusteredLepton>& electronFS = applyProjection<DressedLeptons>(e, "ELECTRON_FS").clusteredLeptons();
			const MissingMomentum& met = applyProjection<MissingMomentum>(e,"MET");		

			vector<ClusteredLepton> dressed_lepton;
			vector<ClusteredLepton> isolated_lepton;
			vector<ClusteredLepton> fiducial_lepton;

			dressed_lepton.insert(dressed_lepton.end(), muonFS.begin(), muonFS.end());		
			dressed_lepton.insert(dressed_lepton.end(), electronFS.begin(), electronFS.end());

			////////////////////////////////////////////////////////////////////////////
			//OVERLAP REMOVAL
			//	-electrons with dR(e,mu)<0.1 are removed
			//	-lower pT electrons with dR(e,e)<0.1 are removed
			//
			////////////////////////////////////////////////////////////////////////////

			foreach(ClusteredLepton &l1,dressed_lepton){

				bool l_isolated = true;

				foreach(ClusteredLepton &l2, dressed_lepton){

					if(l1 != l2 && fabs(l2.constituentLepton().pid()) == PID::ELECTRON){

						double overlapControl_ll= deltaR(l1.constituentLepton(),l2.constituentLepton());

						if(overlapControl_ll < 0.1){

							l_isolated = false;

							// e/e overlap removal
							if(fabs(l1.constituentLepton().pid()) == PID::ELECTRON){

								if(l1.constituentLepton().pT()>l2.constituentLepton().pT()){

									isolated_lepton.push_back(l1);//keep e with highest pT

								}else{

									isolated_lepton.push_back(l2);//keep e with highest pT

								}

							}

							// e/mu overlap removal
							if(l1.constituentLepton().pid() == PID::MUON)	isolated_lepton.push_back(l1);//keep mu

						}								
					}

				}

				if(l_isolated) isolated_lepton.push_back(l1);
			}

			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			//PRESELECTION:
			//"isolated_lepton:"
			//    * electron: pt>20 GeV, |eta|<1.37, 1.52<|eta|<2.47, dR(electron,muon)>0.1
			//    * muon:     pt>20 GeV, |eta|<2.4 
			//		*	dR(l,l)>0.1
			//
			//"fiducial_lepton"= isolated_lepton with
			//				* 2 leptons (e or mu)
			//			  * leading lepton pt (pT_l1) >25 GeV
			//    		* opposite charged leptons
			//
			///////////////////////////////////////////////////////////////////////////////////////////////////////////

			double M_l1l2=0; //dilepton mass
			double pT_l1l2=0; //dilepton pT
			double pT_l1=0; //leading lepton pT

			if(isolated_lepton.size()!=2) vetoEvent;

			sort(isolated_lepton.begin(), isolated_lepton.end(), Particles_pT());

			if(isolated_lepton.at(0).pT()>25*GeV && PID::charge(isolated_lepton.at(0))!=PID::charge(isolated_lepton.at(1))){

			 fiducial_lepton.insert(fiducial_lepton.end(),isolated_lepton.begin(),isolated_lepton.end());

			}

			if(fiducial_lepton.size()==0) vetoEvent;

			pT_l1 = fiducial_lepton.at(0).momentum().pT();
			M_l1l2 = (fiducial_lepton.at(0).momentum() + fiducial_lepton.at(1).momentum()).mass();
			pT_l1l2=(fiducial_lepton.at(0).momentum() + fiducial_lepton.at(1).momentum()).pT();

			/////////////////////////////////////////////////////////////////////////
			//JETS
			//	-"alljets": found by "jetpro" projection && pT()>25 GeV && |y|<4.5
			//	-"vetojets": "alljets"  && dR(electron,jet)>0.3 
			//
			/////////////////////////////////////////////////////////////////////////

			Jets alljets;
			Jets vetojets;

			foreach(const Jet& j, applyProjection<FastJets>(e, "jet").jetsByPt(25)){

				if(fabs(j.momentum().rapidity()) > 4.5 ) continue;

				alljets.push_back(j);
	
				bool deltaRcontrol = true;

				foreach(ClusteredLepton &fl,fiducial_lepton){

					if(fabs(fl.constituentLepton().pid()) == PID::ELECTRON){//electrons

						double deltaRjets = deltaR(fl.constituentLepton().momentum(), j.momentum(),RAPIDITY);

						if(deltaRjets<= 0.3) deltaRcontrol = false;//false if at least one electron is in the overlap region

					}
			
				}
	
				if(deltaRcontrol == true) vetojets.push_back(j);								
			}

			/////////////////////////////////////////////////////////////////////////////////////////////////
			//MISSING ETrel
			//	-"mismom": fourvector of invisible momentum found by "met" projection
			//	-"delta_phi": delta phi between mismom and the nearest "fiducial_lepton" or "vetojet"
			//	-"MET_rel": missing transverse energy defined as:
			//			*"mismom" 	for	"delta_phi" >= (0.5*pi)  
			//			*"mismom.pT()*sin(delta_phi)"	for	"delta_phi" < (0.5*pi)	
			//					
			/////////////////////////////////////////////////////////////////////////////////////////////////

			FourMomentum mismom;
			double MET_rel=0;
			double delta_phi=0;

			mismom = -met.visibleMomentum();

			vector<double> vL_MET_angle;
			vector<double> vJet_MET_angle;

			const double pi = acos(-1);

			vL_MET_angle.push_back(fabs(deltaPhi(fiducial_lepton.at(0).momentum(),mismom)));
			vL_MET_angle.push_back(fabs(deltaPhi(fiducial_lepton.at(1).momentum(),mismom)));

			foreach(double &lM, vL_MET_angle){
				if(lM>pi)	lM=2*pi-lM;						
			}

			std::sort(vL_MET_angle.begin(), vL_MET_angle.end());

			if(vetojets.size()==0) delta_phi = vL_MET_angle.at(0);

			if(vetojets.size()>0){

				foreach(Jet &vj, vetojets){

					double Jet_MET_angle = fabs(deltaPhi(vj.momentum(),mismom));

					if(Jet_MET_angle > pi) Jet_MET_angle=2*pi-Jet_MET_angle;

					vJet_MET_angle.push_back(Jet_MET_angle);

				}
	
				std::sort(vJet_MET_angle.begin(), vJet_MET_angle.end());

				if(vL_MET_angle.at(0) <= vJet_MET_angle.at(0)) delta_phi = vL_MET_angle.at(0);	
				if(vL_MET_angle.at(0) > vJet_MET_angle.at(0)) delta_phi = vJet_MET_angle.at(0);

			}

			if(delta_phi >= (0.5*pi)) delta_phi = (0.5*pi);

			MET_rel = mismom.pT()*sin(delta_phi);
	
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			//CUTS
			//		-jetveto: event with at least one vetojet is vetoed
			//		-M_Z: Z mass M_Z=91.1876*GeV
			//
			//    * ee   channel: MET_rel > 45 GeV, M_l1l2 > 15 GeV, |M_l1l2-M_Z| > 15 GeV, jetveto, pT_l1l2 > 30 GeV
			//    * mumu channel: MET_rel > 45 GeV, M_l1l2 > 15 GeV, |M_l1l2-M_Z| > 15 GeV, jetveto, pT_l1l2 > 30 GeV
			//		* emu  channel: MET_rel > 25 GeV, M_l1l2 > 10 GeV, |M_l1l2-M_Z| > 0  GeV, jetveto, pT_l1l2 > 30 GeV
			//
			////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

			//channel definition
			bool chan_mumu=false;
			bool chan_ee=false;
			bool chan_emu=false;

			if(fabs(fiducial_lepton.at(0).pdgId()) == PID::ELECTRON && fabs(fiducial_lepton.at(1).pdgId()) == PID::ELECTRON) chan_ee=true;
			if(fabs(fiducial_lepton.at(0).pdgId()) == PID::MUON && fabs(fiducial_lepton.at(1).pdgId()) == PID::MUON) chan_mumu=true;
			if(fabs(fiducial_lepton.at(0).pid())!=fabs(fiducial_lepton.at(1).pid())) chan_emu=true;

			//channel cuts
			if(chan_ee == true){

				if(MET_rel <= 45*GeV) vetoEvent;
				if(M_l1l2 <= 15*GeV) vetoEvent;
				if(fabs(M_l1l2 - 91.1876*GeV) <= 15*GeV) vetoEvent;
				if(vetojets.size() != 0) vetoEvent;
				if(pT_l1l2 <= 30*GeV) vetoEvent;

				_h_Wl1_pT_ee->fill(sqrtS()*GeV, weight);
				_h_Wl1_pT_inclusive->fill(pT_l1, weight);

			}

			if(chan_mumu == true){

				if(MET_rel <= 45*GeV) vetoEvent;
				if(M_l1l2 <= 15*GeV) vetoEvent;
				if(fabs(M_l1l2-91.1876*GeV) <= 15*GeV) vetoEvent;
				if(vetojets.size() != 0) vetoEvent;
				if(pT_l1l2 <= 30*GeV) vetoEvent;

				_h_Wl1_pT_mumu->fill(sqrtS()*GeV, weight);
				_h_Wl1_pT_inclusive->fill(pT_l1, weight);	
					
			}

			if(chan_emu == true){

				if(MET_rel <= 25*GeV) vetoEvent;
				if(M_l1l2 <= 10*GeV) vetoEvent;
				if(vetojets.size() != 0) vetoEvent;
				if(pT_l1l2 <= 30*GeV) vetoEvent;

				_h_Wl1_pT_emu->fill(sqrtS()*GeV, weight);
				_h_Wl1_pT_inclusive->fill(pT_l1, weight);

			}
	
		}

		/// Finalize
		void finalize() {

			double norm=crossSection()/sumOfWeights()/femtobarn;

			scale(_h_Wl1_pT_ee, norm);
			scale(_h_Wl1_pT_mumu, norm);
			scale(_h_Wl1_pT_emu, norm);

			normalize(_h_Wl1_pT_inclusive, 1);

		}

		private:

		Histo1DPtr _h_Wl1_pT_ee;
		Histo1DPtr _h_Wl1_pT_mumu;
		Histo1DPtr _h_Wl1_pT_emu;
		Histo1DPtr _h_Wl1_pT_inclusive;

  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2013_I1190187);

}
